import _ from "underscore";

function addValue(tinymceConfig, option, expectedValue) {
	// option string is empty or undefined
	if (!tinymceConfig[option]) {
		tinymceConfig[option] = expectedValue;
		return;
	}

	// check if option already contains expectedValue
	var options = tinymceConfig[option].split(" ");
	if (options.indexOf(expectedValue) > -1) {
		return;
	}

	// append expectedValue to string
	if (tinymceConfig[option].substr(tinymceConfig[option].length - 1) !== " ") {
		tinymceConfig[option] += " ";
	}
	tinymceConfig[option] += expectedValue;
}

export default function(options) {
	options = _.defaults(options || {}, {
		allowTextAlign: true
	});

	return function attachRTLPlugin(tinymceConfig) {
		addValue(tinymceConfig, "plugins", "directionality");
		addValue(tinymceConfig, "plugins", "paste");
		addValue(tinymceConfig, "paste_retain_style_properties", "direction");
		if (options.allowTextAlign) {
			addValue(tinymceConfig, "paste_retain_style_properties", "text-align");
		}
		addValue(tinymceConfig, "paste_webkit_styles", "direction");
		if (options.allowTextAlign) {
			addValue(tinymceConfig, "paste_webkit_styles", "text-align");
		}

		var originalPastePostProcess = tinymceConfig.paste_postprocess;
		tinymceConfig.paste_postprocess = function(plugin, args) {
			// Convert top-level children from 'span' to 'p' tags and copy all innerhtml and attributes
			_.each(args.node.childNodes, function(child) {
				if (child.tagName === "SPAN") {
					var newElement = document.createElement("p");
					newElement.innerHTML = child.innerHTML;
					_.each(child.attributes, function(attribute) {
						newElement.setAttribute(attribute.name, attribute.value);
					});
					args.node.replaceChild(newElement, child);
				}
			});

			// Convert 'direction' css style to global 'dir' attribute
			var directionStyleElements = args.node.querySelectorAll("[style*=direction]");
			_.each(directionStyleElements, function(el) {
				el.dir = el.style.direction;
				el.style.direction = "";
			});

			// Remove 'text-align' and 'align' properties from nodes that have a 'dir' attribute
			var directionAttrElements = args.node.querySelectorAll("[dir]");
			_.each(directionAttrElements, function(el) {
				el.dir = el.dir.toLowerCase();
				el.style.textAlign = "";
				el.removeAttribute("align");
			});

			// Continue paste_postprocess event call
			if (originalPastePostProcess) {
				originalPastePostProcess(plugin, args);
			}
		};

		return tinymceConfig;
	};
}
